@@ -84,7 +84,7 @@ class QSConnectionAsync:
         await self.disconnect()
 
     async def disconnect(self):
-        await self.run_command("QUIT")
+        await self._protocol.disconnect()
         self._transport.close()
         self.connected = False
 
@@ -119,7 +119,7 @@ class QSConnectionAsync:
     async def connect(
         self,
         authenticate: Optional[bool] = None,
-        initial_access_level: Literal[AccessLevel, None] = None,
+        initial_access_level: AccessLevel | None = None,
         password: Optional[str] = None,
     ) -> str:
 
@@ -240,16 +240,19 @@ class QSConnectionAsync:
             filterset_r = filterset
 
         fl = await self.get_exp_file(
-            f"{run}/apldbio/sds/filter/S{stage:02}_C{cycle:03}"
+            f'"{run}/apldbio/sds/filter/S{stage:02}_C{cycle:03}'
             f"_T{step:02}_P{point:04}_M{filterset_r.em}"
-            f"_X{filterset_r.ex}_filterdata.xml"
+            f'_X{filterset_r.ex}_filterdata.xml"'
         )
 
-        f = data.FilterDataReading(ET.parse(io.BytesIO(fl)).find("PlatePointData/PlateData"))
+        if (x := ET.parse(io.BytesIO(fl)).find("PlatePointData/PlateData")) is not None:
+            f = data.FilterDataReading(x)
+        else:
+            raise ValueError("PlateData not found")
 
         ql = (
             await self.get_expfile_list(
-                f"{run}/apldbio/sds/quant/" f"{f.filename_reading_string}_E*.quant"
+                f'"{run}/apldbio/sds/quant/' f'{f.filename_reading_string}_E*.quant"'
             )
         )[-1]
         qf = await self.get_exp_file(ql)
