@@ -33,12 +33,13 @@ from copy import deepcopy
 log = logging.getLogger(__name__)
 
 
-def _temperature_str(temperatures: Union[float, Iterable[float]]) -> str:
+def _temperature_str(temperatures: Union[float, Iterable[float], np.ndarray]) -> str:
     """Tries to do the right thing in generating a string for a temperature or
     list of temperatures."""
     if isinstance(temperatures, Iterable):
         temperatures = list(temperatures)
-        if len(temperatures) == 1 or len(set(temperatures)) == 1:
+        unique = np.unique(temperatures)
+        if len(temperatures) == 1 or len(unique) == 1:
             return f"{temperatures[0]:.02f}°C"
         else:
             return "[" + ", ".join(f"{x:.02f}" for x in temperatures) + "]" + "°C"
@@ -408,7 +409,7 @@ class Protocol(XMLable):
         The stages of the protocol, likely :any:`Stage`.
     name: str | None
         A protocol name. If not set, a timestamp will be used, unlike AB's uuid.
-    volume: floa
+    volume: float
         The sample volume, in µL.
     runmode: str | None
         The run mode.
@@ -711,10 +712,10 @@ class HACFILT(ProtoCommand):
     filters: Sequence[FilterSet] | None
 
     def to_command(
-        self, filters: Iterable[FilterSet | str] | None = None, **kwargs
+        self, filters: Sequence[FilterSet | str] | None = None, **kwargs
     ) -> str:
-        if self.filters is None:
-            assert filters is not None
+        if self.filters is None or len(self.filters) == 0:
+            assert (filters is not None) and len(filters) > 0
             filters = [
                 FilterSet.fromstring(f) if isinstance(f, str) else f for f in filters
             ]
